@@ -11,36 +11,42 @@ Module for managing Windows Users
 :depends:
         - pywintypes
         - win32api
+        - win32con
         - win32net
         - win32netcon
         - win32profile
         - win32security
         - win32ts
+        - wmi
 
 .. note::
     This currently only works with local user accounts, not domain accounts
 '''
-from __future__ import absolute_import
-from datetime import datetime
+# Import Python libs
+from __future__ import absolute_import, unicode_literals, print_function
+import logging
 import time
+from datetime import datetime
 
 try:
     from shlex import quote as _cmd_quote  # pylint: disable=E0611
-except:  # pylint: disable=W0702
+except Exception:  # pylint: disable=broad-except
     from pipes import quote as _cmd_quote
 
-# Import salt libs
-import salt.utils
+# Import Salt libs
+import salt.utils.args
+import salt.utils.dateutils
+import salt.utils.platform
+import salt.utils.winapi
+from salt.ext import six
 from salt.ext.six import string_types
 from salt.exceptions import CommandExecutionError
-import logging
 
 log = logging.getLogger(__name__)
 
 try:
     import pywintypes
     import wmi
-    import pythoncom
     import win32api
     import win32con
     import win32net
@@ -58,69 +64,41 @@ __virtualname__ = 'user'
 
 def __virtual__():
     '''
-    Set the user module if the kernel is Windows
+    Requires Windows and Windows Modules
     '''
-    if HAS_WIN32NET_MODS and salt.utils.is_windows():
-        return __virtualname__
-    return False
+    if not salt.utils.platform.is_windows():
+        return False, 'Module win_useradd: Windows Only'
+
+    if not HAS_WIN32NET_MODS:
+        return False, 'Module win_useradd: Missing Win32 Modules'
+
+    return __virtualname__
 
 
-def _get_date_time_format(dt_string):
+def _to_unicode(instr):
     '''
-    Copied from win_system.py (_get_date_time_format)
+    Internal function for converting to Unicode Strings
 
-    Function that detects the date/time format for the string passed.
+    The NetUser* series of API calls in this module requires input parameters to
+    be Unicode Strings. This function ensures the parameter is a Unicode String.
+    This only seems to be an issue in Python 2. All calls to this function
+    should be gated behind a ``if six.PY2`` check.
 
-    :param str dt_string:
-        A date/time string
+    Args:
+        instr (str): String to convert
 
-    :return: The format of the passed dt_string
-    :rtype: str
+    Returns:
+        str: Unicode type string
     '''
-    valid_formats = [
-        '%Y-%m-%d %I:%M:%S %p',
-        '%m-%d-%y %I:%M:%S %p',
-        '%m-%d-%Y %I:%M:%S %p',
-        '%m/%d/%y %I:%M:%S %p',
-        '%m/%d/%Y %I:%M:%S %p',
-        '%Y/%m/%d %I:%M:%S %p',
-        '%Y-%m-%d %I:%M:%S',
-        '%m-%d-%y %I:%M:%S',
-        '%m-%d-%Y %I:%M:%S',
-        '%m/%d/%y %I:%M:%S',
-        '%m/%d/%Y %I:%M:%S',
-        '%Y/%m/%d %I:%M:%S',
-        '%Y-%m-%d %I:%M %p',
-        '%m-%d-%y %I:%M %p',
-        '%m-%d-%Y %I:%M %p',
-        '%m/%d/%y %I:%M %p',
-        '%m/%d/%Y %I:%M %p',
-        '%Y/%m/%d %I:%M %p',
-        '%Y-%m-%d %I:%M',
-        '%m-%d-%y %I:%M',
-        '%m-%d-%Y %I:%M',
-        '%m/%d/%y %I:%M',
-        '%m/%d/%Y %I:%M',
-        '%Y/%m/%d %I:%M',
-        '%Y-%m-%d',
-        '%m-%d-%y',
-        '%m-%d-%Y',
-        '%m/%d/%y',
-        '%m/%d/%Y',
-        '%Y/%m/%d',
-    ]
-    for dt_format in valid_formats:
-        try:
-            datetime.strptime(dt_string, dt_format)
-            return dt_format
-        except ValueError:
-            continue
-    return False
+    if instr is None or isinstance(instr, six.text_type):
+        return instr
+    else:
+        return six.text_type(instr, 'utf-8')
 
 
 def add(name,
         password=None,
-        fullname=False,
+        fullname=None,
         description=None,
         groups=None,
         home=None,
@@ -130,38 +108,32 @@ def add(name,
     '''
     Add a user to the minion.
 
-    :param str name:
-        User name
+    Args:
+        name (str): User name
 
-    :param str password:
-        User's password in plain text.
+        password (str, optional): User's password in plain text.
 
-    :param str fullname:
-        The user's full name.
+        fullname (str, optional): The user's full name.
 
-    :param str description:
-        A brief description of the user account.
+        description (str, optional): A brief description of the user account.
 
-    :param list groups:
-        A list of groups to add the user to.
+        groups (str, optional): A list of groups to add the user to.
+            (see chgroups)
 
-    :param str home:
-        The path to the user's home directory.
+        home (str, optional): The path to the user's home directory.
 
-    :param str homedrive:
-        The drive letter to assign to the home directory. Must be the Drive Letter
-        followed by a colon. ie: U:
+        homedrive (str, optional): The drive letter to assign to the home
+            directory. Must be the Drive Letter followed by a colon. ie: U:
 
-    :param str profile:
-        An explicit path to a profile. Can be a UNC or a folder on the system. If
-        left blank, windows uses it's default profile directory.
+        profile (str, optional): An explicit path to a profile. Can be a UNC or
+            a folder on the system. If left blank, windows uses it's default
+            profile directory.
 
-    :param str logonscript:
-        Path to a login script to run when the user logs on.
+        logonscript (str, optional): Path to a login script to run when the user
+            logs on.
 
-    :return:
-        True if successful. False is unsuccessful.
-    :rtype: bool
+    Returns:
+        bool: True if successful. False is unsuccessful.
 
     CLI Example:
 
@@ -169,6 +141,16 @@ def add(name,
 
         salt '*' user.add name password
     '''
+    if six.PY2:
+        name = _to_unicode(name)
+        password = _to_unicode(password)
+        fullname = _to_unicode(fullname)
+        description = _to_unicode(description)
+        home = _to_unicode(home)
+        homedrive = _to_unicode(homedrive)
+        profile = _to_unicode(profile)
+        logonscript = _to_unicode(logonscript)
+
     user_info = {}
     if name:
         user_info['name'] = name
@@ -184,11 +166,10 @@ def add(name,
     try:
         win32net.NetUserAdd(None, 1, user_info)
     except win32net.error as exc:
-        (number, context, message) = exc
-        log.error('Failed to create user {0}'.format(name))
-        log.error('nbr: {0}'.format(number))
-        log.error('ctx: {0}'.format(context))
-        log.error('msg: {0}'.format(message))
+        log.error('Failed to create user %s', name)
+        log.error('nbr: %s', exc.winerror)
+        log.error('ctx: %s', exc.funcname)
+        log.error('msg: %s', exc.strerror)
         return False
 
     update(name=name,
@@ -215,80 +196,83 @@ def update(name,
            unlock_account=None,
            password_never_expires=None,
            disallow_change_password=None):
-    r'''
+    # pylint: disable=anomalous-backslash-in-string
+    '''
     Updates settings for the windows user. Name is the only required parameter.
     Settings will only be changed if the parameter is passed a value.
 
     .. versionadded:: 2015.8.0
 
-    :param str name:
-        The user name to update.
+    Args:
+        name (str): The user name to update.
 
-    :param str password:
-        New user password in plain text.
+        password (str, optional): New user password in plain text.
 
-    :param str fullname:
-        The user's full name.
+        fullname (str, optional): The user's full name.
 
-    :param str description:
-        A brief description of the user account.
+        description (str, optional): A brief description of the user account.
 
-    :param str home:
-        The path to the user's home directory.
+        home (str, optional): The path to the user's home directory.
 
-    :param str homedrive:
-        The drive letter to assign to the home directory. Must be the Drive Letter
-        followed by a colon. ie: U:
+        homedrive (str, optional): The drive letter to assign to the home
+            directory. Must be the Drive Letter followed by a colon. ie: U:
 
-    :param str logonscript:
-        The path to the logon script.
+        logonscript (str, optional): The path to the logon script.
 
-    :param str profile:
-        The path to the user's profile directory.
+        profile (str, optional): The path to the user's profile directory.
 
-    :param date expiration_date: The date and time when the account expires. Can
-    be a valid date/time string. To set to never expire pass the string 'Never'.
+        expiration_date (date, optional): The date and time when the account
+            expires. Can be a valid date/time string. To set to never expire
+            pass the string 'Never'.
 
-    :param bool expired: Pass `True` to expire the account. The user will be
-    prompted to change their password at the next logon. Pass `False` to mark
-    the account as 'not expired'. You can't use this to negate the expiration if
-    the expiration was caused by the account expiring. You'll have to change
-    the `expiration_date` as well.
+        expired (bool, optional): Pass `True` to expire the account. The user
+            will be prompted to change their password at the next logon. Pass
+            `False` to mark the account as 'not expired'. You can't use this to
+            negate the expiration if the expiration was caused by the account
+            expiring. You'll have to change the `expiration_date` as well.
 
-    :param bool account_disabled: True disables the account. False enables the
-    account.
+        account_disabled (bool, optional): True disables the account. False
+            enables the account.
 
-    :param bool unlock_account: True unlocks a locked user account. False is
-    ignored.
+        unlock_account (bool, optional): True unlocks a locked user account.
+            False is ignored.
 
-    :param bool password_never_expires: True sets the password to never expire.
-    False allows the password to expire.
+        password_never_expires (bool, optional): True sets the password to never
+            expire. False allows the password to expire.
 
-    :param bool disallow_change_password: True blocks the user from changing
-    the password. False allows the user to change the password.
+        disallow_change_password (bool, optional): True blocks the user from
+            changing the password. False allows the user to change the password.
 
-    :return:
-        True if successful. False is unsuccessful.
-    :rtype: bool
+    Returns:
+        bool: True if successful. False is unsuccessful.
 
     CLI Example:
 
     .. code-block:: bash
 
-        salt '*' user.update bob password=secret profile=C:\Users\Bob
+        salt '*' user.update bob password=secret profile=C:\\Users\\Bob
                  home=\\server\homeshare\bob homedrive=U:
     '''
+    # pylint: enable=anomalous-backslash-in-string
+    if six.PY2:
+        name = _to_unicode(name)
+        password = _to_unicode(password)
+        fullname = _to_unicode(fullname)
+        description = _to_unicode(description)
+        home = _to_unicode(home)
+        homedrive = _to_unicode(homedrive)
+        logonscript = _to_unicode(logonscript)
+        profile = _to_unicode(profile)
 
     # Make sure the user exists
     # Return an object containing current settings for the user
     try:
         user_info = win32net.NetUserGetInfo(None, name, 4)
     except win32net.error as exc:
-        (number, context, message) = exc
-        log.error('Failed to update user {0}'.format(name))
-        log.error('nbr: {0}'.format(number))
-        log.error('ctx: {0}'.format(context))
-        log.error('msg: {0}'.format(message))
+        log.error('Failed to update user %s', name)
+        log.error('nbr: %s', exc.winerror)
+        log.error('ctx: %s', exc.funcname)
+        log.error('msg: %s', exc.strerror)
         return False
 
     # Check parameters to update
@@ -311,11 +295,10 @@ def update(name,
         if expiration_date == 'Never':
             user_info['acct_expires'] = win32netcon.TIMEQ_FOREVER
         else:
-            date_format = _get_date_time_format(expiration_date)
-            if date_format:
-                dt_obj = datetime.strptime(expiration_date, date_format)
-            else:
-                return 'Invalid start_date'
+            try:
+                dt_obj = salt.utils.dateutils.date_cast(expiration_date)
+            except (ValueError, RuntimeError):
+                return 'Invalid Date/Time Format: {0}'.format(expiration_date)
             user_info['acct_expires'] = time.mktime(dt_obj.timetuple())
     if expired is not None:
         if expired:
@@ -326,30 +309,29 @@ def update(name,
         if account_disabled:
             user_info['flags'] |= win32netcon.UF_ACCOUNTDISABLE
         else:
-            user_info['flags'] ^= win32netcon.UF_ACCOUNTDISABLE
+            user_info['flags'] &= ~win32netcon.UF_ACCOUNTDISABLE
     if unlock_account is not None:
         if unlock_account:
-            user_info['flags'] ^= win32netcon.UF_LOCKOUT
+            user_info['flags'] &= ~win32netcon.UF_LOCKOUT
     if password_never_expires is not None:
         if password_never_expires:
             user_info['flags'] |= win32netcon.UF_DONT_EXPIRE_PASSWD
         else:
-            user_info['flags'] ^= win32netcon.UF_DONT_EXPIRE_PASSWD
+            user_info['flags'] &= ~win32netcon.UF_DONT_EXPIRE_PASSWD
     if disallow_change_password is not None:
         if disallow_change_password:
             user_info['flags'] |= win32netcon.UF_PASSWD_CANT_CHANGE
         else:
-            user_info['flags'] ^= win32netcon.UF_PASSWD_CANT_CHANGE
+            user_info['flags'] &= ~win32netcon.UF_PASSWD_CANT_CHANGE
 
     # Apply new settings
     try:
         win32net.NetUserSetInfo(None, name, 4, user_info)
     except win32net.error as exc:
-        (number, context, message) = exc
-        log.error('Failed to update user {0}'.format(name))
-        log.error('nbr: {0}'.format(number))
-        log.error('ctx: {0}'.format(context))
-        log.error('msg: {0}'.format(message))
+        log.error('Failed to update user %s', name)
+        log.error('nbr: %s', exc.winerror)
+        log.error('ctx: %s', exc.funcname)
+        log.error('msg: %s', exc.strerror)
         return False
 
     return True
@@ -361,20 +343,19 @@ def delete(name,
     '''
     Remove a user from the minion
 
-    :param str name:
-        The name of the user to delete
+    Args:
+        name (str): The name of the user to delete
 
-    :param bool purge:
-        Boolean value indicating that the user profile should also be removed when
-        the user account is deleted. If set to True the profile will be removed.
+        purge (bool, optional): Boolean value indicating that the user profile
+            should also be removed when the user account is deleted. If set to
+            True the profile will be removed. Default is False.
 
-    :param bool force:
-        Boolean value indicating that the user account should be deleted even if the
-        user is logged in. True will log the user out and delete user.
+        force (bool, optional): Boolean value indicating that the user account
+            should be deleted even if the user is logged in. True will log the
+            user out and delete user.
 
-    :return:
-        True if successful
-    :rtype: bool
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -382,15 +363,17 @@ def delete(name,
 
         salt '*' user.delete name
     '''
+    if six.PY2:
+        name = _to_unicode(name)
+
     # Check if the user exists
     try:
         user_info = win32net.NetUserGetInfo(None, name, 4)
     except win32net.error as exc:
-        (number, context, message) = exc
-        log.error('User not found: {0}'.format(name))
-        log.error('nbr: {0}'.format(number))
-        log.error('ctx: {0}'.format(context))
-        log.error('msg: {0}'.format(message))
+        log.error('User not found: %s', name)
+        log.error('nbr: %s', exc.winerror)
+        log.error('ctx: %s', exc.funcname)
+        log.error('msg: %s', exc.strerror)
         return False
 
     # Check if the user is logged in
@@ -398,11 +381,10 @@ def delete(name,
     try:
         sess_list = win32ts.WTSEnumerateSessions()
     except win32ts.error as exc:
-        (number, context, message) = exc
         log.error('No logged in users found')
-        log.error('nbr: {0}'.format(number))
-        log.error('ctx: {0}'.format(context))
-        log.error('msg: {0}'.format(message))
+        log.error('nbr: %s', exc.winerror)
+        log.error('ctx: %s', exc.funcname)
+        log.error('msg: %s', exc.strerror)
 
     # Is the user one that is logged in
     logged_in = False
@@ -419,14 +401,13 @@ def delete(name,
             try:
                 win32ts.WTSLogoffSession(win32ts.WTS_CURRENT_SERVER_HANDLE, session_id, True)
             except win32ts.error as exc:
-                (number, context, message) = exc
-                log.error('User not found: {0}'.format(name))
-                log.error('nbr: {0}'.format(number))
-                log.error('ctx: {0}'.format(context))
-                log.error('msg: {0}'.format(message))
+                log.error('User not found: %s', name)
+                log.error('nbr: %s', exc.winerror)
+                log.error('ctx: %s', exc.funcname)
+                log.error('msg: %s', exc.strerror)
                 return False
         else:
-            log.error('User {0} is currently logged in.'.format(name))
+            log.error('User %s is currently logged in.', name)
             return False
 
     # Remove the User Profile directory
@@ -435,25 +416,24 @@ def delete(name,
             sid = getUserSid(name)
             win32profile.DeleteProfile(sid)
         except pywintypes.error as exc:
-            (number, context, message) = exc
+            (number, context, message) = exc.args
             if number == 2:  # Profile Folder Not Found
                 pass
             else:
-                log.error('Failed to remove profile for {0}'.format(name))
-                log.error('nbr: {0}'.format(number))
-                log.error('ctx: {0}'.format(context))
-                log.error('msg: {0}'.format(message))
+                log.error('Failed to remove profile for %s', name)
+                log.error('nbr: %s', exc.winerror)
+                log.error('ctx: %s', exc.funcname)
+                log.error('msg: %s', exc.strerror)
                 return False
 
     # And finally remove the user account
     try:
         win32net.NetUserDel(None, name)
     except win32net.error as exc:
-        (number, context, message) = exc
-        log.error('Failed to delete user {0}'.format(name))
-        log.error('nbr: {0}'.format(number))
-        log.error('ctx: {0}'.format(context))
-        log.error('msg: {0}'.format(message))
+        log.error('Failed to delete user %s', name)
+        log.error('nbr: %s', exc.winerror)
+        log.error('ctx: %s', exc.funcname)
+        log.error('msg: %s', exc.strerror)
         return False
 
     return True
@@ -463,12 +443,11 @@ def getUserSid(username):
     '''
     Get the Security ID for the user
 
-    :param str username:
-        user name for which to look up the SID
+    Args:
+        username (str): The user name for which to look up the SID
 
-    :return:
-        Returns the user SID
-    :rtype: str
+    Returns:
+        str: The user SID
 
     CLI Example:
 
@@ -476,27 +455,29 @@ def getUserSid(username):
 
         salt '*' user.getUserSid jsnuffy
     '''
+    if six.PY2:
+        username = _to_unicode(username)
+
     domain = win32api.GetComputerName()
-    if username.find(u'\\') != -1:
-        domain = username.split(u'\\')[0]
-        username = username.split(u'\\')[-1]
+    if username.find('\\') != -1:
+        domain = username.split('\\')[0]
+        username = username.split('\\')[-1]
     domain = domain.upper()
-    return win32security.ConvertSidToStringSid(win32security.LookupAccountName(None, domain + u'\\' + username)[0])
+    return win32security.ConvertSidToStringSid(
+        win32security.LookupAccountName(None, domain + '\\' + username)[0])
 
 
 def setpassword(name, password):
     '''
     Set the user's password
 
-    :param str name:
-        user name for which to set the password
+    Args:
+        name (str): The user name for which to set the password
 
-    :param str password:
-        the new password
+        password (str): The new password
 
-    :return:
-        True if successful. False is unsuccessful.
-    :rtype: bool
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -511,15 +492,13 @@ def addgroup(name, group):
     '''
     Add user to a group
 
-    :param str name:
-        user name to add to the group
+    Args:
+        name (str): The user name to add to the group
 
-    :param str group:
-        name of the group to which to add the user
+        group (str): The name of the group to which to add the user
 
-    :return:
-        True if successful. False is unsuccessful.
-    :rtype: bool
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -527,6 +506,10 @@ def addgroup(name, group):
 
         salt '*' user.addgroup jsnuffy 'Power Users'
     '''
+    if six.PY2:
+        name = _to_unicode(name)
+        group = _to_unicode(group)
+
     name = _cmd_quote(name)
     group = _cmd_quote(group).lstrip('\'').rstrip('\'')
 
@@ -546,15 +529,13 @@ def removegroup(name, group):
     '''
     Remove user from a group
 
-    :param str name:
-        user name to remove from the group
+    Args:
+        name (str): The user name to remove from the group
 
-    :param str group:
-        name of the group from which to remove the user
+        group (str): The name of the group from which to remove the user
 
-    :return:
-        True if successful. False is unsuccessful.
-    :rtype: bool
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -562,6 +543,10 @@ def removegroup(name, group):
 
         salt '*' user.removegroup jsnuffy 'Power Users'
     '''
+    if six.PY2:
+        name = _to_unicode(name)
+        group = _to_unicode(group)
+
     name = _cmd_quote(name)
     group = _cmd_quote(group).lstrip('\'').rstrip('\'')
 
@@ -579,23 +564,18 @@ def removegroup(name, group):
     return ret['retcode'] == 0
 
 
-def chhome(name, home, persist=False):
+def chhome(name, home, **kwargs):
     '''
     Change the home directory of the user, pass True for persist to move files
     to the new home directory if the old home directory exist.
 
-    :param str name:
-        name of the user whose home directory you wish to change
+    Args:
+        name (str): The name of the user whose home directory you wish to change
 
-    :param str home:
-        new location of the home directory
+        home (str): The new location of the home directory
 
-    :param bool persist:
-        True to move the contents of the existing home directory to the new location
-
-    :return:
-        True if successful. False is unsuccessful.
-    :rtype: bool
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -603,6 +583,17 @@ def chhome(name, home, persist=False):
 
         salt '*' user.chhome foo \\\\fileserver\\home\\foo True
     '''
+    if six.PY2:
+        name = _to_unicode(name)
+        home = _to_unicode(home)
+
+    kwargs = salt.utils.args.clean_kwargs(**kwargs)
+    persist = kwargs.pop('persist', False)
+    if kwargs:
+        salt.utils.args.invalid_kwargs(kwargs)
+    if persist:
+        log.info('Ignoring unsupported \'persist\' argument to user.chhome')
+
     pre_info = info(name)
 
     if not pre_info:
@@ -614,11 +605,6 @@ def chhome(name, home, persist=False):
     if not update(name=name, home=home):
         return False
 
-    if persist and home is not None and pre_info['home'] is not None:
-        cmd = 'move /Y {0} {1}'.format(pre_info['home'], home)
-        if __salt__['cmd.retcode'](cmd, python_shell=False) != 0:
-            log.debug('Failed to move the contents of the Home Directory')
-
     post_info = info(name)
     if post_info['home'] != pre_info['home']:
         return post_info['home'] == home
@@ -630,15 +616,13 @@ def chprofile(name, profile):
     '''
     Change the profile directory of the user
 
-    :param str name:
-        name of the user whose profile you wish to change
+    Args:
+        name (str): The name of the user whose profile you wish to change
 
-    :param str profile:
-        new location of the profile
+        profile (str): The new location of the profile
 
-    :return:
-    True if successful. False is unsuccessful.
-    :rtype: bool
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -653,15 +637,13 @@ def chfullname(name, fullname):
     '''
     Change the full name of the user
 
-    :param str name:
-        user name for which to change the full name
+    Args:
+        name (str): The user name for which to change the full name
 
-    :param str fullname:
-        the new value for the full name
+        fullname (str): The new value for the full name
 
-    :return:
-        True if successful. False is unsuccessful.
-    :rtype: bool
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -677,20 +659,19 @@ def chgroups(name, groups, append=True):
     Change the groups this user belongs to, add append=False to make the user a
     member of only the specified groups
 
-    :param str name:
-        user name for which to change groups
+    Args:
+        name (str): The user name for which to change groups
 
-    :param groups:
-        a single group or a list of groups to assign to the user
-    :type groups: list, str
+        groups (str, list): A single group or a list of groups to assign to the
+            user. For multiple groups this can be a comma delimited string or a
+            list.
 
-    :param bool append:
-        True adds the passed groups to the user's current groups
-        False sets the user's groups to the passed groups only
+        append (bool, optional): True adds the passed groups to the user's
+            current groups. False sets the user's groups to the passed groups
+            only. Default is True.
 
-    :return:
-        True if successful. False is unsuccessful.
-    :rtype: bool
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -698,10 +679,16 @@ def chgroups(name, groups, append=True):
 
         salt '*' user.chgroups jsnuffy Administrators,Users True
     '''
+    if six.PY2:
+        name = _to_unicode(name)
+
     if isinstance(groups, string_types):
         groups = groups.split(',')
 
     groups = [x.strip(' *') for x in groups]
+    if six.PY2:
+        groups = [_to_unicode(x) for x in groups]
+
     ugrps = set(list_groups(name))
     if ugrps == set(groups):
         return True
@@ -720,7 +707,10 @@ def chgroups(name, groups, append=True):
             continue
         group = _cmd_quote(group).lstrip('\'').rstrip('\'')
         cmd = 'net localgroup "{0}" {1} /add'.format(group, name)
-        __salt__['cmd.run_all'](cmd, python_shell=True)
+        out = __salt__['cmd.run_all'](cmd, python_shell=True)
+        if out['retcode'] != 0:
+            log.error(out['stdout'])
+            return False
 
     agrps = set(list_groups(name))
     return len(ugrps - agrps) == 0
@@ -730,11 +720,11 @@ def info(name):
     '''
     Return user information
 
-    :param str name:
-        Username for which to display information
+    Args:
+        name (str): Username for which to display information
 
-    :returns:
-        A dictionary containing user information
+    Returns:
+        dict: A dictionary containing user information
             - fullname
             - username
             - SID
@@ -756,7 +746,6 @@ def info(name):
             - password_never_expires
             - disallow_change_password
             - gid
-    :rtype: dict
 
     CLI Example:
 
@@ -764,6 +753,9 @@ def info(name):
 
         salt '*' user.info jsnuffy
     '''
+    if six.PY2:
+        name = _to_unicode(name)
+
     ret = {}
     items = {}
     try:
@@ -830,20 +822,31 @@ def info(name):
 
     else:
 
-        return False
+        return {}
 
 
 def _get_userprofile_from_registry(user, sid):
     '''
-    In case net user doesn't return the userprofile
-    we can get it from the registry
+    In case net user doesn't return the userprofile we can get it from the
+    registry
+
+    Args:
+        user (str): The user name, used in debug message
+
+        sid (str): The sid to lookup in the registry
+
+    Returns:
+        str: Profile directory
     '''
     profile_dir = __salt__['reg.read_value'](
         'HKEY_LOCAL_MACHINE',
-        u'SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\ProfileList\\{0}'.format(sid),
+        'SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\ProfileList\\{0}'.format(sid),
         'ProfileImagePath'
     )['vdata']
-    log.debug(u'user {0} with sid={2} profile is located at "{1}"'.format(user, profile_dir, sid))
+    log.debug(
+        'user %s with sid=%s profile is located at "%s"',
+        user, sid, profile_dir
+    )
     return profile_dir
 
 
@@ -851,12 +854,11 @@ def list_groups(name):
     '''
     Return a list of groups the named user belongs to
 
-    :param str name:
-        user name for which to list groups
+    Args:
+        name (str): The user name for which to list groups
 
-    :return:
-        list of groups to which the user belongs
-    :rtype: list
+    Returns:
+        list: A list of groups to which the user belongs
 
     CLI Example:
 
@@ -864,6 +866,9 @@ def list_groups(name):
 
         salt '*' user.list_groups foo
     '''
+    if six.PY2:
+        name = _to_unicode(name)
+
     ugrp = set()
     try:
         user = info(name)['groups']
@@ -879,13 +884,12 @@ def getent(refresh=False):
     '''
     Return the list of all info for all users
 
-    :param bool refresh:
-        Refresh the cached user information. Default is False. Useful when used from
-        within a state function.
+    Args:
+        refresh (bool, optional): Refresh the cached user information. Useful
+            when used from within a state function. Default is False.
 
-    :return:
-        A dictionary containing information about all users on the system
-    :rtype: dict
+    Returns:
+        dict: A dictionary containing information about all users on the system
 
     CLI Example:
 
@@ -917,11 +921,10 @@ def getent(refresh=False):
 
 def list_users():
     '''
-    Return a list of users on Windows
+    Return a list of all users on Windows
 
-    :return:
-        list of users on the system
-    :rtype: list
+    Returns:
+        list: A list of all users on the system
 
     CLI Example:
 
@@ -930,7 +933,6 @@ def list_users():
         salt '*' user.list_users
     '''
     res = 0
-    users = []
     user_list = []
     dowhile = True
     try:
@@ -954,15 +956,13 @@ def rename(name, new_name):
     '''
     Change the username for a named user
 
-    :param str name:
-        user name to change
+    Args:
+        name (str): The user name to change
 
-    :param str new_name:
-        the new name for the current user
+        new_name (str): The new name for the current user
 
-    :return:
-        True if successful. False is unsuccessful.
-    :rtype: bool
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -970,26 +970,32 @@ def rename(name, new_name):
 
         salt '*' user.rename jsnuffy jshmoe
     '''
+    if six.PY2:
+        name = _to_unicode(name)
+        new_name = _to_unicode(new_name)
+
     # Load information for the current name
     current_info = info(name)
     if not current_info:
-        raise CommandExecutionError('User {0!r} does not exist'.format(name))
+        raise CommandExecutionError('User \'{0}\' does not exist'.format(name))
 
     # Look for an existing user with the new name
     new_info = info(new_name)
     if new_info:
-        raise CommandExecutionError('User {0!r} already exists'.format(new_name))
+        raise CommandExecutionError(
+            'User \'{0}\' already exists'.format(new_name)
+        )
 
     # Rename the user account
     # Connect to WMI
-    pythoncom.CoInitialize()
-    c = wmi.WMI(find_classes=0)
+    with salt.utils.winapi.Com():
+        c = wmi.WMI(find_classes=0)
 
     # Get the user object
     try:
         user = c.Win32_UserAccount(Name=name)[0]
     except IndexError:
-        raise CommandExecutionError('User {0!r} does not exist'.format(name))
+        raise CommandExecutionError('User \'{0}\' does not exist'.format(name))
 
     # Rename the user
     result = user.Rename(new_name)[0]
@@ -1008,16 +1014,12 @@ def rename(name, new_name):
                       8: 'Operation is not allowed on specified special groups: user, admin, local, or guest',
                       9: 'Other API error',
                       10: 'Internal error'}
-        raise CommandExecutionError('There was an error renaming {0!r} to {1!r}. Error: {2}'.format(name, new_name, error_dict[result]))
+        raise CommandExecutionError(
+            'There was an error renaming \'{0}\' to \'{1}\'. Error: {2}'
+            .format(name, new_name, error_dict[result])
+        )
 
-    # Load information for the new name
-    post_info = info(new_name)
-
-    # Verify that the name has changed
-    if post_info['name'] != current_info['name']:
-        return post_info['name'] == new_name
-
-    return False
+    return info(new_name).get('name') == new_name
 
 
 def current(sam=False):
@@ -1029,16 +1031,13 @@ def current(sam=False):
 
     .. versionadded:: 2015.5.6
 
-    :param bool sam:
-        False returns just the username without any domain notation. True
-        returns the domain with the username in the SAM format. Ie:
+    Args:
+        sam (bool, optional): False returns just the username without any domain
+            notation. True returns the domain with the username in the SAM
+            format. Ie: ``domain\\username``
 
-        ``domain\\username``
-
-    :return:
-        Returns False if the username cannot be returned. Otherwise returns the
-        username.
-    :rtype: bool str
+    Returns:
+        str: Returns username
 
     CLI Example:
 
@@ -1052,14 +1051,13 @@ def current(sam=False):
         else:
             user_name = win32api.GetUserName()
     except pywintypes.error as exc:
-        (number, context, message) = exc
         log.error('Failed to get current user')
-        log.error('nbr: {0}'.format(number))
-        log.error('ctx: {0}'.format(context))
-        log.error('msg: {0}'.format(message))
-        return False
+        log.error('nbr: %s', exc.winerror)
+        log.error('ctx: %s', exc.funcname)
+        log.error('msg: %s', exc.strerror)
+        raise CommandExecutionError('Failed to get current user', info=exc)
 
     if not user_name:
-        return False
+        raise CommandExecutionError('Failed to get current user')
 
     return user_name
