@@ -12,8 +12,6 @@ from tests.support.unit import skipIf, TestCase
 from tests.support.mock import (
     MagicMock,
     Mock,
-    NO_MOCK,
-    NO_MOCK_REASON,
     patch,
     call
 )
@@ -37,7 +35,85 @@ def _docker_py_version():
     return (0,)
 
 
-@skipIf(NO_MOCK, NO_MOCK_REASON)
+class DockerUnitTestCase(TestCase, LoaderModuleMockMixin):
+    def fake_run(self, *args, **kwargs):
+        print(args, kwargs)
+        return '{}'
+
+    def setup_loader_modules(self):
+        return {
+            docker_mod: {
+                '__utils__': {
+                    'state.get_sls_opts': MagicMock(return_value={
+                        'pillarenv': MagicMock(),
+                        'pillar': {},
+                        'grains': {},
+                    }),
+                    'args.clean_kwargs': lambda **x: x,
+                },
+                '__salt__': {
+                    'config.option': MagicMock(return_value=None),
+                    'cmd.run': self.fake_run,
+                },
+                '__opts__': {
+                    'id': 'dockermod-unit-test',
+                },
+            },
+        }
+
+    def test_trans_tar_should_have_grains_in_sls_opts_including_pillar_override(self):
+        container_name = 'fnord'
+        expected_grains = {
+            'roscivs': 'bottia',
+            'fnord': 'dronf',
+            'salt': 'NaCl',
+        }
+        expected_pillars = {
+            'this': {
+                'is': {
+                    'my': {
+                        'pillar': 'data',
+                    },
+                },
+            },
+        }
+        extra_pillar_data = {'some': 'extras'}
+        fake_trans_tar = MagicMock(return_value=b'hi')
+        patch_trans_tar = patch(
+            'salt.modules.dockermod._prepare_trans_tar',
+            fake_trans_tar,
+        )
+        patch_call = patch(
+            'salt.modules.dockermod.call',
+            MagicMock(return_value=expected_grains),
+        )
+        fake_get_pillar = MagicMock()
+        fake_get_pillar.compile_pillar.return_value = expected_pillars
+        patch_pillar = patch(
+            'salt.modules.dockermod.salt.pillar.get_pillar',
+            MagicMock(return_value=fake_get_pillar),
+        )
+        patch_run_all = patch(
+            'salt.modules.dockermod.run_all',
+            MagicMock(return_value={'retcode': 1, 'stderr': 'early exit test'}),
+        )
+        with patch_trans_tar, patch_call, patch_pillar, patch_run_all:
+            docker_mod.sls(container_name, pillar=extra_pillar_data)
+            # TODO: It would be fine if we could make this test require less magic numbers -W. Werner, 2019-08-27
+            actual_sls_opts = fake_trans_tar.call_args[0][1]
+            self.assertDictContainsSubset(
+                expected_grains,
+                actual_sls_opts['grains'],
+                'Docker container grains not provided to thin client creation',
+            )
+            expected_pillars.update(extra_pillar_data)
+            self.assertDictContainsSubset(
+                expected_pillars,
+                actual_sls_opts['pillar'],
+                'Docker container pillar not provided to thin client creation',
+            )
+
+
 @skipIf(docker_mod.HAS_DOCKER_PY is False, 'docker-py must be installed to run these tests. Skipping.')
 class DockerTestCase(TestCase, LoaderModuleMockMixin):
     '''
@@ -45,7 +121,7 @@ class DockerTestCase(TestCase, LoaderModuleMockMixin):
     '''
     def setup_loader_modules(self):
         utils = salt.loader.utils(
-            salt.config.DEFAULT_MINION_OPTS,
+            salt.config.DEFAULT_MINION_OPTS.copy(),
             whitelist=['args', 'docker', 'json', 'state', 'thin',
                        'systemd', 'path', 'platform']
         )
@@ -78,7 +154,12 @@ class DockerTestCase(TestCase, LoaderModuleMockMixin):
         with patch.dict(docker_mod.__pillar__, {'docker-registries': {'portus.example.com:5000':
                 {'username': 'admin', 'password': 'linux12345', 'email': 'tux@example.com'}}}):
             with patch.object(docker_mod, '_get_client', get_client_mock):
-                with patch.dict(docker_mod.__salt__, {'cmd.run_all': MagicMock(return_value=ref_out)}):
+                dunder_salt = {
+                    'cmd.run_all': MagicMock(return_value=ref_out),
+                    'config.get': MagicMock(return_value={}),
+                    'config.option': MagicMock(return_value={}),
+                }
+                with patch.dict(docker_mod.__salt__, dunder_salt):
                     ret = docker_mod.login('portus.example.com:5000')
                     self.assertIn('retcode', ret)
                     self.assertNotEqual(ret['retcode'], 0)
@@ -168,13 +249,20 @@ class DockerTestCase(TestCase, LoaderModuleMockMixin):
                                 {'mine.send': mine_send,
                                  'container_resource.run': MagicMock(),
                                  'config.get': MagicMock(return_value=True),
+                                 'config.option': MagicMock(return_value=True),
                                  'cp.cache_file': MagicMock(return_value=False)}):
                     with patch.dict(docker_mod.__utils__,
                                     {'docker.get_client_args': client_args_mock}):
                         with patch.object(docker_mod, '_get_client', client):
                             command('container', *args)
-                mine_send.assert_called_with('docker.ps', verbose=True, all=True,
-                                             host=True)
+                try:
+                    mine_send.assert_called_with(
+                        'docker.ps', verbose=True, all=True, host=True)
+                except AssertionError as exc:
+                    raise Exception(
+                        'command \'{0}\' did not call docker.ps with expected '
+                        'arguments: {1}'.format(command_name, exc)
+                    )
 
     def test_update_mine(self):
         '''
@@ -199,6 +287,7 @@ class DockerTestCase(TestCase, LoaderModuleMockMixin):
         mine_mock = Mock()
         dunder_salt = {
             'config.get': MagicMock(side_effect=config_get_disabled),
+            'config.option': MagicMock(return_value=False),
             'mine.send': mine_mock,
         }
         with patch.dict(docker_mod.__salt__, dunder_salt), \
@@ -211,6 +300,7 @@ class DockerTestCase(TestCase, LoaderModuleMockMixin):
                 patch.dict(docker_mod.__context__, {'docker.client': Mock()}), \
                 patch.object(docker_mod, 'state', MagicMock(return_value='stopped')):
             dunder_salt['config.get'].side_effect = config_get_enabled
+            dunder_salt['config.option'].return_value = True
             docker_mod.stop('foo', timeout=1)
             self.assert_called_once(mine_mock)
 
@@ -838,7 +928,7 @@ class DockerTestCase(TestCase, LoaderModuleMockMixin):
         inspect_container_mock = MagicMock(side_effect=_inspect_container_effect)
 
         with patch.object(docker_mod, 'inspect_container', inspect_container_mock):
-            ret = docker_mod.compare_container('container1', 'container2')
+            ret = docker_mod.compare_container('container1', 'container2')  # pylint: disable=not-callable
             self.assertEqual(ret, {})
 
     def test_compare_container_env_order(self):
@@ -867,7 +957,7 @@ class DockerTestCase(TestCase, LoaderModuleMockMixin):
         inspect_container_mock = MagicMock(side_effect=_inspect_container_effect)
 
         with patch.object(docker_mod, 'inspect_container', inspect_container_mock):
-            ret = docker_mod.compare_container('container1', 'container2')
+            ret = docker_mod.compare_container('container1', 'container2')  # pylint: disable=not-callable
             self.assertEqual(ret, {})
 
     def test_resolve_tag(self):
